@@ -56,7 +56,6 @@ struct _ECalBackendGroupwisePrivate {
 	GHashTable *categories_by_id;
 	GHashTable *categories_by_name;
 
-
 	/* fields for storing info while offline */
 	char *user_email;
 };
@@ -115,8 +114,10 @@ populate_cache (ECalBackendGroupwise *cbgw)
         GList *list = NULL, *l;
 	gboolean done = FALSE;
 	int cursor = 0;
+	icalcomponent_kind kind;
 	
 	priv = cbgw->priv;
+	kind = e_cal_backend_get_kind (E_CAL_BACKEND (cbgw));
 	
 	if (!mutex) {
 		mutex = g_mutex_new ();
@@ -138,9 +139,15 @@ populate_cache (ECalBackendGroupwise *cbgw)
                 return status;
         }
 	
+	status = e_gw_connection_position_cursor (priv->cnc, priv->container_id, cursor, "end", 1);
+	if (status != E_GW_CONNECTION_STATUS_OK) {
+		e_cal_backend_groupwise_notify_error_code (cbgw, status);
+		g_mutex_unlock (mutex);
+                return status;
+        }
 	while (!done) {
 		
-		status = e_gw_connection_read_cursor (priv->cnc, priv->container_id, cursor, 1, CURSOR_ITEM_LIMIT, &list);
+		status = e_gw_connection_read_cursor (priv->cnc, priv->container_id, cursor, FALSE, CURSOR_ITEM_LIMIT, &list);
 		if (status != E_GW_CONNECTION_STATUS_OK) {
 			e_cal_backend_groupwise_notify_error_code (cbgw, status);
 			g_mutex_unlock (mutex);
@@ -153,7 +160,14 @@ populate_cache (ECalBackendGroupwise *cbgw)
 			comp = e_gw_item_to_cal_component (item, cbgw);
 			g_object_unref (item);
 			if (E_IS_CAL_COMPONENT (comp)) {
+				char *comp_str;
+				
 				e_cal_component_commit_sequence (comp);
+				if (kind == icalcomponent_isa (e_cal_component_get_icalcomponent (comp))) {
+					comp_str = e_cal_component_get_as_string (comp);	
+					e_cal_backend_notify_object_created (E_CAL_BACKEND (cbgw), (const char *) comp_str);
+					g_free (comp_str);
+				}
 				e_cal_backend_cache_put_component (priv->cache, comp);
 				g_object_unref (comp);
 			}
@@ -175,61 +189,151 @@ static gboolean
 get_deltas (gpointer handle)
 {
  	ECalBackendGroupwise *cbgw;
+	ECalBackendGroupwisePrivate *priv;
 	EGwConnection *cnc; 
  	ECalBackendCache *cache; 
         EGwConnectionStatus status; 
-	GSList *deletes = NULL, *updates = NULL, *adds = NULL, *l;
+	GSList *item_list, *cache_keys, *l;
+	const char *cache_file_name;
+	char *comp_str;
+	static time_t mod_time = 0;
+	GTimeVal time_val;
+	char time_string[100];
+	const struct tm *tm;
+	struct stat buf;
         
 	if (!handle)
 		return FALSE;
 	
 	cbgw = (ECalBackendGroupwise *) handle;
- 	cnc = cbgw->priv->cnc; 
- 	cache = cbgw->priv->cache; 
+	priv= cbgw->priv;
+ 	cnc = priv->cnc; 
+ 	cache = priv->cache; 
+	item_list = NULL;
+	
+	if (!mod_time) {
+		cache_file_name = e_file_cache_get_filename (E_FILE_CACHE (priv->cache));
+		printf ("%s %d\n", cache_file_name, stat (cache_file_name, &buf));
+		mod_time = buf.st_mtime;
+	}
 
-	/* Call e-gw-connection_get_deltas*/
-	status = e_gw_connection_get_deltas (cnc, &adds, &deletes, &updates);
+	tm = gmtime (&mod_time);
+	strftime (time_string, 100, "%Y-%m-%dT%H:%M:%SZ", tm);
+	
+	status = e_gw_connection_get_quick_messages (cnc, cbgw->priv->container_id, "recipients message recipientStatus default", time_string, "New", "CalendarItem", NULL,  -1,  &item_list);
+	if (status != E_GW_CONNECTION_STATUS_OK) {
+		e_cal_backend_groupwise_notify_error_code (cbgw, status);
+		return TRUE;
+	}
+	for (; item_list != NULL; item_list = g_slist_next(item_list)) {
+		EGwItem *item = E_GW_ITEM(item_list->data);
+		ECalComponent *comp = e_gw_item_to_cal_component (item, cbgw);
+		
+		e_cal_component_commit_sequence (comp);
+
+		if (comp) {
+			if (!e_cal_backend_cache_put_component (cache, comp)) 
+				g_message ("Could not add the component");
+			else  {
+				comp_str = e_cal_component_get_as_string (comp);	
+				e_cal_backend_notify_object_created (E_CAL_BACKEND (cbgw), comp_str);
+				g_free (comp_str);
+			}
+		}
+		else 
+			g_message ("Invalid component returned");
+
+		g_object_unref (comp);
+		g_object_unref (item);
+	}
+	if (item_list) {
+		g_slist_free (item_list);
+		item_list = NULL;
+	}
+	status = e_gw_connection_get_quick_messages (cnc, cbgw->priv->container_id,"recipients message recipientStatus  default", time_string, "Modified", "CalendarItem", NULL,  -1,  &item_list);
+	
+	if (status != E_GW_CONNECTION_STATUS_OK) {
+		e_cal_backend_groupwise_notify_error_code (cbgw, status);
+		return TRUE;
+	}
+	
+	for (; item_list != NULL; item_list = g_slist_next(item_list)) {
+		EGwItem *item = E_GW_ITEM(item_list->data);
+		ECalComponent *modified_comp, *cache_comp;
+		char *cache_comp_str;
+		
+		modified_comp = e_gw_item_to_cal_component (item, cbgw);
+		if (!modified_comp) {
+			g_message ("Invalid component returned in update");
+			continue;
+		}
+		cache_comp = e_cal_backend_cache_get_component (cache, e_gw_item_get_icalid (item), NULL);
+		e_cal_component_commit_sequence (modified_comp);
+		e_cal_component_commit_sequence (cache_comp);
+
+		cache_comp_str = e_cal_component_get_as_string (cache_comp);
+		e_cal_backend_notify_object_modified (E_CAL_BACKEND (cbgw), cache_comp_str, e_cal_component_get_as_string (modified_comp));
+		e_cal_backend_cache_remove_component (cache, e_gw_item_get_icalid (item), NULL);
+		e_cal_backend_cache_put_component (cache, modified_comp);
+		g_free (cache_comp_str);
+		g_object_unref (item);
+		g_object_unref (modified_comp);
+	}
+	if (item_list) {
+		g_slist_free (item_list);
+		item_list = NULL;
+	}
+
+	status = e_gw_connection_get_quick_messages (cnc, cbgw->priv->container_id,"iCalId", NULL, "All", "CalendarItem", NULL,  -1,  &item_list);
 
 	if (status != E_GW_CONNECTION_STATUS_OK) {
 		e_cal_backend_groupwise_notify_error_code (cbgw, status);
-		return FALSE;
+		return TRUE;
+	}
+
+	/* handle deleted items here by going over the entire cache and
+	 * checking for deleted items.*/
+	
+	cache_keys = e_cal_backend_cache_get_keys (cache);
+	for (l = item_list; l; l = g_slist_next (l)) {
+		/* this works assuming rid is null*/
+		cache_keys = g_slist_delete_link (cache_keys, 
+				g_slist_find_custom (cache_keys, l->data, (GCompareFunc) strcmp));
+		g_free (l->data);
 	}
 	
-	if (deletes) {
-		for (l = deletes; l != NULL; l = g_slist_next(l)) {
-			if (!e_cal_backend_cache_remove_component (cache, (char *)l->data, NULL)) 
-				g_message ("Could not remove %s", (char *)l->data);
-		}
-	}
+	for (l = cache_keys; l ; l = g_slist_next (l)) {
+		/* assumes rid is null - which works for now */
+		ECalComponent *comp = NULL;
+		ECalComponentVType vtype;
 
-	if (adds) {
-		for (l = adds; l != NULL; l = g_slist_next (l)) {
-			EGwItem *item = (EGwItem *) l->data;
-			ECalComponent *comp = e_gw_item_to_cal_component (item, cbgw);
-			if (!comp)
-				g_message ("Invalid component returned");
-			else if (!e_cal_backend_cache_put_component (cache, comp)) 
-				g_message ("Could not add the component");
-		}
-	}	
-
-	if (updates) {
-		for (l = updates; l != NULL; l = g_slist_next (l)) {
-			EGwItem *item = (EGwItem *) l->data;
-			ECalComponent *comp = e_cal_backend_cache_get_component (cache, e_gw_item_get_id (item), NULL);
-			if (!comp) /* FIXME  Error in updates. Skipping the element*/
-				continue;
-			/* FIXME  currently, just overwrite the fields with the
-			 * update.*/
-			e_cal_backend_cache_remove_component (cache, e_gw_item_get_id (item), NULL);
-			e_cal_backend_cache_put_component (cache, e_gw_item_to_cal_component (item, cbgw));
+		comp = e_cal_backend_cache_get_component (cache, (const char *) l->data, NULL);	
+		vtype = e_cal_component_get_vtype (comp);
+		if ((vtype == E_CAL_COMPONENT_EVENT) ||
+				(vtype == E_CAL_COMPONENT_TODO)) {
+			comp_str = e_cal_component_get_as_string (comp);
+			e_cal_backend_notify_object_removed (E_CAL_BACKEND (cbgw), 
+								(char *) l->data, comp_str);
+			e_cal_backend_cache_remove_component (cache, (const char *) l->data, NULL);
+			g_free (comp_str);
 		}
+		g_object_unref (comp);
 	}
+	if (item_list) {
+		g_slist_free (item_list);
+		item_list = NULL;
+	}
+	if (cache_keys) {
+		g_slist_free (cache_keys);
+		item_list = NULL;
+	}
+		
+	g_get_current_time (&time_val);
+	mod_time = time_val.tv_sec;
 
         return TRUE;        
 }
 
-
 static char* 
 form_uri (ESource *source)
 {
@@ -264,17 +368,85 @@ form_uri (ESource *source)
 
 }
 
+static ECalBackendSyncStatus
+cache_init (ECalBackendGroupwise *cbgw)
+{
+	ECalBackendGroupwisePrivate *priv = cbgw->priv;
+	EGwConnectionStatus cnc_status;
+	icalcomponent_kind kind;
+	const char *time_interval_string;
+	int time_interval;
 
+	kind = e_cal_backend_get_kind (E_CAL_BACKEND (cbgw));
+	
+	time_interval = CACHE_REFRESH_INTERVAL;
+	time_interval_string = g_getenv ("GETQM_TIME_INTERVAL");
+	if (time_interval_string) {
+		time_interval = g_ascii_strtod (time_interval_string, NULL);
+		time_interval *= (60*1000); 
+		
+	}
+	
+	/* We poke the cache for a default timezone. Its
+	 * absence indicates that the cache file has not been
+	 * populated before. */
+	if (!e_cal_backend_cache_get_marker (priv->cache)) {
+		/* Populate the cache for the first time.*/
+		/* start a timed polling thread set to 1 minute*/
+		cnc_status = populate_cache (cbgw);
+		if (cnc_status != E_GW_CONNECTION_STATUS_OK) {
+			g_warning (G_STRLOC ": Could not populate the cache");
+			/*FIXME  why dont we do a notify here */
+			return GNOME_Evolution_Calendar_PermissionDenied;
+		} else {
+			e_cal_backend_cache_set_marker (priv->cache);
+			priv->timeout_id = g_timeout_add (time_interval, (GSourceFunc) get_deltas, (gpointer) cbgw);
+			priv->mode = CAL_MODE_REMOTE;
+			return GNOME_Evolution_Calendar_Success;
+		}
+	} else {
+		GList *cache_items = NULL, *l;
+		/* notify the ecal about the objects already in cache */
+		cache_items = e_cal_backend_cache_get_components (priv->cache);
+		
+		for (l = cache_items; l; l = g_list_next (l)) {
+			ECalComponent *comp = E_CAL_COMPONENT (l->data);
+			char *cal_string;
+
+			if (kind == icalcomponent_isa (e_cal_component_get_icalcomponent (comp))) {
+				cal_string = e_cal_component_get_as_string (comp);
+				e_cal_backend_notify_object_created (E_CAL_BACKEND (cbgw), cal_string);
+				g_free (cal_string);
+			}
+			g_object_unref (comp);
+		}		
+		if (cache_items)
+			g_list_free (cache_items);
+		
+		/* get the deltas from the cache */
+		if (get_deltas (cbgw)) {
+			priv->timeout_id = g_timeout_add (time_interval, (GSourceFunc) get_deltas, (gpointer) cbgw);
+			priv->mode = CAL_MODE_REMOTE;
+			return GNOME_Evolution_Calendar_Success;
+		} else {
+			g_warning (G_STRLOC ": Could not populate the cache");
+			/*FIXME  why dont we do a notify here */
+			return GNOME_Evolution_Calendar_PermissionDenied;	
+		}
+	}
+	
+}
 
 static ECalBackendSyncStatus
 connect_to_server (ECalBackendGroupwise *cbgw)
 {
 	char *real_uri;
 	ECalBackendGroupwisePrivate *priv;
-	EGwConnectionStatus cnc_status;
 	ESource *source;
 	const char *use_ssl;
 	char *http_uri;
+	GThread *thread;
+	GError *error = NULL;
 	priv = cbgw->priv;
 
 	source = e_cal_backend_get_source (E_CAL_BACKEND (cbgw));
@@ -327,39 +499,24 @@ connect_to_server (ECalBackendGroupwise *cbgw)
 			} else
 				priv->container_id = NULL;
 
-				priv->cache = e_cal_backend_cache_new (e_cal_backend_get_uri (E_CAL_BACKEND (cbgw)));
-				if (!priv->cache) {
-					g_mutex_unlock (priv->mutex);
-					e_cal_backend_notify_error (E_CAL_BACKEND (cbgw), _("Could not create cache file"));
-					return GNOME_Evolution_Calendar_OtherError;
-				}
-			
-			/* read the default timezone*/
-			priv->default_zone = e_cal_backend_cache_get_default_timezone (priv->cache);
-
-			/* Clear the cache before populating it */
-			e_file_cache_clean (E_FILE_CACHE (priv->cache));
-
-			if (priv->default_zone)
-				e_cal_backend_cache_put_default_timezone (priv->cache, priv->default_zone);
-	
-			/* Populate the cache for the first time.*/
-			/* start a timed polling thread set to 10 minutes*/
-			cnc_status = populate_cache (cbgw);
-			if (cnc_status != E_GW_CONNECTION_STATUS_OK) {
-				g_object_unref (priv->cnc);
-				g_object_unref (priv->cache);
-				priv->cnc = NULL;
-				priv->cache = NULL;
-				g_warning (G_STRLOC ": Could not populate the cache");
-				return GNOME_Evolution_Calendar_PermissionDenied;
-			} else {
-				g_object_ref (priv->cnc);
-				g_object_ref (priv->cache);
-				priv->timeout_id = g_timeout_add (CACHE_REFRESH_INTERVAL, (GSourceFunc) get_deltas, (gpointer) cbgw);
-				priv->mode = CAL_MODE_REMOTE;
-				return GNOME_Evolution_Calendar_Success;
+			priv->cache = e_cal_backend_cache_new (e_cal_backend_get_uri (E_CAL_BACKEND (cbgw)));
+			if (!priv->cache) {
+				g_mutex_unlock (priv->mutex);
+				e_cal_backend_notify_error (E_CAL_BACKEND (cbgw), _("Could not create cache file"));
+				return GNOME_Evolution_Calendar_OtherError;
 			}
+
+			/* spawn a new thread for opening the calendar */
+			thread = g_thread_create ((GThreadFunc) cache_init, cbgw, FALSE, &error);
+			if (!thread) {
+				g_warning (G_STRLOC ": %s", error->message);
+				g_error_free (error);
+
+				e_cal_backend_notify_error (E_CAL_BACKEND (cbgw), _("Could not create thread for populating cache"));
+				return GNOME_Evolution_Calendar_OtherError;
+			}
+
+			
 		} else {
 			e_cal_backend_notify_error (E_CAL_BACKEND (cbgw), _("Authentication failed"));
 			return GNOME_Evolution_Calendar_AuthenticationFailed;
@@ -512,13 +669,13 @@ e_cal_backend_groupwise_get_alarm_email_address (ECalBackendSync *backend, EData
 static ECalBackendSyncStatus
 e_cal_backend_groupwise_get_static_capabilities (ECalBackendSync *backend, EDataCal *cal, char **capabilities)
 {
-	*capabilities = g_strdup (CAL_STATIC_CAPABILITY_NO_EMAIL_ALARMS ","
-				  CAL_STATIC_CAPABILITY_ONE_ALARM_ONLY ","
-				  CAL_STATIC_CAPABILITY_REMOVE_ALARMS ","
-	                          CAL_STATIC_CAPABILITY_NO_THISANDPRIOR ","
-				  CAL_STATIC_CAPABILITY_NO_THISANDFUTURE ","
-				  CAL_STATIC_CAPABILITY_NO_CONV_TO_ASSIGN_TASK ","
-				  CAL_STATIC_CAPABILITY_NO_CONV_TO_RECUR ","
+	*capabilities = g_strdup (CAL_STATIC_CAPABILITY_NO_EMAIL_ALARMS "," \
+				  CAL_STATIC_CAPABILITY_ONE_ALARM_ONLY "," \
+				  CAL_STATIC_CAPABILITY_REMOVE_ALARMS ","   \
+	                          CAL_STATIC_CAPABILITY_NO_THISANDPRIOR "," \
+				  CAL_STATIC_CAPABILITY_NO_THISANDFUTURE "," \
+				  CAL_STATIC_CAPABILITY_NO_CONV_TO_ASSIGN_TASK "," \
+				  CAL_STATIC_CAPABILITY_NO_CONV_TO_RECUR "," \
 				  CAL_STATIC_CAPABILITY_SAVE_SCHEDULES);
 
 	return GNOME_Evolution_Calendar_Success;
@@ -1086,7 +1243,7 @@ e_cal_backend_groupwise_create_object (ECalBackendSync *backend, EDataCal *cal,
 
 	comp = e_cal_component_new ();
 	e_cal_component_set_icalcomponent (comp, icalcomp);
-
+	
 	/* check if the object exists */
 	switch (priv->mode) {
 	case CAL_MODE_ANY :
@@ -1095,7 +1252,11 @@ e_cal_backend_groupwise_create_object (ECalBackendSync *backend, EDataCal *cal,
 		status = e_gw_connection_create_appointment (priv->cnc, priv->container_id, cbgw, comp, &uid_list);
 		if (status != E_GW_CONNECTION_STATUS_OK) {
 			g_object_unref (comp);
-			return GNOME_Evolution_Calendar_OtherError;
+
+			if (status == E_GW_CONNECTION_STATUS_UNKNOWN_USER)
+				return GNOME_Evolution_Calendar_UnknownUser;
+			else
+				return GNOME_Evolution_Calendar_OtherError;
 		}
 	
 		if (uid_list && (g_slist_length (uid_list) == 1)) {
@@ -1116,6 +1277,9 @@ e_cal_backend_groupwise_create_object (ECalBackendSync *backend, EDataCal *cal,
 			/* convert uid_list to GPtrArray and get the items in a list */
 			e_gw_connection_get_items_from_ids (priv->cnc, priv->container_id, "recipients message",
 					uid_array, &list);
+			/* FIXME  check if list is null and status may have
+			 * failed. */
+			comp = g_object_ref ( (ECalComponent *) list->data );
 			/* convert items into components and add them to the cache */
 			for (i=0, tmp = list; tmp ; tmp = g_list_next (tmp), i++) {
 				ECalComponent *e_cal_comp;
@@ -1134,7 +1298,7 @@ e_cal_backend_groupwise_create_object (ECalBackendSync *backend, EDataCal *cal,
 				if (i != 0) {
 					char *temp;
 					temp = e_cal_component_get_as_string (e_cal_comp);
-					e_cal_backend_notify_object_created (E_CAL_BACKEND (cbgw), g_strdup(temp));
+					e_cal_backend_notify_object_created (E_CAL_BACKEND (cbgw), temp);
 					g_free (temp);
 				}
 
@@ -1204,6 +1368,7 @@ e_cal_backend_groupwise_modify_object (ECalBackendSync *backend, EDataCal *cal,
 					g_object_unref (cache_comp);
 					return GNOME_Evolution_Calendar_OtherError;
 				}
+				e_cal_backend_cache_put_component (priv->cache, comp);
 				break;
 			}
 		}
@@ -1327,7 +1492,7 @@ receive_object (ECalBackendGroupwise *cbgw, EDataCal *cal, icalcomponent *icalco
 	e_cal_component_set_icalcomponent (comp, icalcomponent_new_clone (icalcomp));
 	method = icalcomponent_get_method (icalcomp);
 	
-	status = e_gw_connection_send_appointment (priv->cnc, priv->container_id, comp, method, &remove);
+	status = e_gw_connection_send_appointment (cbgw, priv->container_id, comp, method, &remove, &modif_comp);
 
 	if (status == E_GW_CONNECTION_STATUS_OK && !modif_comp)
 		return GNOME_Evolution_Calendar_Success;
@@ -1340,24 +1505,27 @@ receive_object (ECalBackendGroupwise *cbgw, EDataCal *cal, icalcomponent *icalco
 			e_cal_component_get_uid (comp, (const char **) &uid);
 			e_cal_backend_cache_remove_component (priv->cache, uid, NULL);
 			e_cal_backend_notify_object_removed (E_CAL_BACKEND (cbgw), uid, e_cal_component_get_as_string (comp));
+			g_free (comp);
 		}
 		else {
-			char *cache_comp = NULL, *modif_comp, *temp;
+			char *cache_comp = NULL, *temp;
 			ECalComponent *cache_component;
 			
-			e_cal_component_get_uid (comp, &temp);	
+			e_cal_component_commit_sequence (modif_comp);
+			e_cal_component_get_uid (modif_comp, (const char **) &temp);	
 			cache_component = e_cal_backend_cache_get_component (priv->cache, temp, NULL);
 			
-			if (cache_component)
+			if (cache_component) {
+				e_cal_component_commit_sequence (cache_component);
 				cache_comp = e_cal_component_get_as_string (cache_component);
+			}
 
-			e_cal_backend_cache_put_component (priv->cache, comp);	
-			modif_comp = e_cal_component_get_as_string (comp);
+			e_cal_backend_cache_put_component (priv->cache, modif_comp);	
 			
 			if (cache_comp)
-				e_cal_backend_notify_object_modified (E_CAL_BACKEND (cbgw), cache_comp, modif_comp);
+				e_cal_backend_notify_object_modified (E_CAL_BACKEND (cbgw), cache_comp, e_cal_component_get_as_string (modif_comp));
 			else
-				e_cal_backend_notify_object_created (E_CAL_BACKEND (cbgw), e_cal_component_get_as_string (comp));
+				e_cal_backend_notify_object_created (E_CAL_BACKEND (cbgw), e_cal_component_get_as_string (modif_comp));
 				
 			g_free (cache_comp);
 			g_free (modif_comp);
