@@ -89,6 +89,8 @@ convert_status (const GNOME_Evolution_Calendar_CallStatus status)
 		return E_CALENDAR_STATUS_AUTHENTICATION_FAILED;
 	case GNOME_Evolution_Calendar_AuthenticationRequired:
 		return E_CALENDAR_STATUS_AUTHENTICATION_REQUIRED;
+	case GNOME_Evolution_Calendar_UnknownUser:
+		return E_CALENDAR_STATUS_UNKNOWN_USER;
 	case GNOME_Evolution_Calendar_OtherError:
 	default:
 		return E_CALENDAR_STATUS_OTHER_ERROR;
