@@ -29,6 +29,7 @@
 
 struct _ECalBackendCachePrivate {
 	char *uri;
+	GHashTable *timezones;
 };
 
 /* Property IDs */
@@ -111,6 +112,13 @@ e_cal_backend_cache_get_property (GObject *object, guint property_id, GValue *va
 	}
 }
 
+static void
+free_timezone_hash (gpointer key, gpointer value, gpointer user_data)
+{
+	g_free (key);
+	icaltimezone_free (value, 1);
+}
+
 static void
 e_cal_backend_cache_finalize (GObject *object)
 {
@@ -126,6 +134,12 @@ e_cal_backend_cache_finalize (GObject *object)
 			priv->uri = NULL;
 		}
 
+		if (priv->timezones) {
+			g_hash_table_foreach (priv->timezones, (GHFunc) free_timezone_hash, NULL);
+			g_hash_table_destroy (priv->timezones);
+			priv->timezones = NULL;
+		}
+
 		g_free (priv);
 		cache->priv = NULL;
 	}
@@ -184,7 +198,10 @@ e_cal_backend_cache_init (ECalBackendCache *cache)
 	ECalBackendCachePrivate *priv;
 
 	priv = g_new0 (ECalBackendCachePrivate, 1);
+	priv->timezones = g_hash_table_new (g_str_hash, g_str_equal);
+
 	cache->priv = priv;
+
 }
 
 /**
@@ -380,9 +397,15 @@ e_cal_backend_cache_get_components (ECalBackendCache *cache)
                 if (comp_str) {
                         icalcomp = icalparser_parse_string (comp_str);
                         if (icalcomp) {
-                                comp = e_cal_component_new ();
-                                e_cal_component_set_icalcomponent (comp, icalcomp);
-                                list = g_list_append (list, comp);
+				icalcomponent_kind kind;
+
+				kind = icalcomponent_isa (icalcomp);
+				if (kind == ICAL_VEVENT_COMPONENT || kind == ICAL_VTODO_COMPONENT) {
+					comp = e_cal_component_new ();
+					e_cal_component_set_icalcomponent (comp, icalcomp);
+					list = g_list_append (list, comp);
+				} else
+					icalcomponent_free (icalcomp);
                         }
                 }
                 
@@ -390,3 +413,196 @@ e_cal_backend_cache_get_components (ECalBackendCache *cache)
 
         return list;
 }
+
+const icaltimezone *
+e_cal_backend_cache_get_timezone (ECalBackendCache *cache, const char *tzid)
+{
+	icaltimezone *zone;
+	const char *comp_str;
+	ECalBackendCachePrivate *priv;
+
+	g_return_val_if_fail (E_IS_CAL_BACKEND_CACHE (cache), NULL);
+	g_return_val_if_fail (tzid != NULL, NULL);
+
+	priv = cache->priv;
+
+	/* we first look for the timezone in the timezones hash table */
+	zone = g_hash_table_lookup (priv->timezones, tzid);
+	if (zone)
+		return (const icaltimezone *) zone;
+
+	/* if not found look for the timezone in the cache */
+	comp_str = e_file_cache_get_object (E_FILE_CACHE (cache), tzid);
+	if (comp_str) {
+		icalcomponent *icalcomp;
+
+		icalcomp = icalparser_parse_string (comp_str);
+		if (icalcomp) {
+			zone = icaltimezone_new ();
+			if (icaltimezone_set_component (zone, icalcomp) == 1)
+				g_hash_table_insert (priv->timezones, g_strdup (tzid), zone);
+			else {
+				icalcomponent_free (icalcomp);
+				icaltimezone_free (zone, 1);
+			}
+		}
+	}
+
+	return (const icaltimezone *) zone;
+}
+
+gboolean
+e_cal_backend_cache_put_timezone (ECalBackendCache *cache, const icaltimezone *zone)
+{
+	ECalBackendCachePrivate *priv;
+	gpointer orig_key, orig_value;
+	icaltimezone *new_zone;
+	icalcomponent *icalcomp;
+	gboolean retval;
+
+	g_return_val_if_fail (E_IS_CAL_BACKEND_CACHE (cache), FALSE);
+	g_return_val_if_fail (zone != NULL, FALSE);
+
+	priv = cache->priv;
+
+	/* add the timezone to the cache file */
+	icalcomp = icaltimezone_get_component (zone);
+	if (!icalcomp)
+		return FALSE;
+
+	if (e_file_cache_get_object (E_FILE_CACHE (cache), icaltimezone_get_tzid (zone))) {
+		retval = e_file_cache_replace_object (E_FILE_CACHE (cache),
+						      icaltimezone_get_tzid (zone),
+						      icalcomponent_as_ical_string (icalcomp));
+	} else {
+		retval = e_file_cache_add_object (E_FILE_CACHE (cache),
+						  icaltimezone_get_tzid (zone),
+						  icalcomponent_as_ical_string (icalcomp));
+	}
+
+	if (!retval)
+		return FALSE;
+
+	/* check if the timezone already exists */
+	if (g_hash_table_lookup_extended (priv->timezones, icaltimezone_get_tzid (zone),
+					  &orig_key, &orig_value)) {
+		/* remove the previous timezone */
+		g_hash_table_remove (priv->timezones, orig_key);
+		g_free (orig_key);
+		icaltimezone_free (orig_value, 1);
+	}
+
+	/* add the timezone to the hash table */
+	new_zone = icaltimezone_new ();
+	icaltimezone_set_component (new_zone, icalcomponent_new_clone (icalcomp));
+	g_hash_table_insert (priv->timezones, g_strdup (icaltimezone_get_tzid (new_zone)), new_zone);
+
+	return TRUE;
+}
+
+gboolean
+e_cal_backend_cache_put_default_timezone (ECalBackendCache *cache, icaltimezone *default_zone)
+{
+	ECalBackendCachePrivate *priv;
+	icalcomponent *icalcomp;
+	gboolean retval;
+
+	g_return_val_if_fail (E_IS_CAL_BACKEND_CACHE (cache), FALSE);
+
+	priv = cache->priv;
+
+	/* add the timezone to the cache file */
+	icalcomp = icaltimezone_get_component (default_zone);
+	if (!icalcomp)
+		return FALSE;
+
+	if (e_file_cache_get_object (E_FILE_CACHE (cache), "default_zone")) {
+		retval = e_file_cache_replace_object (E_FILE_CACHE (cache), "default_zone",
+						      icalcomponent_as_ical_string (icalcomp));
+	} else {
+		retval = e_file_cache_add_object (E_FILE_CACHE (cache),
+						 "default_zone",
+						  icalcomponent_as_ical_string (icalcomp));
+	}
+
+	if (!retval)
+		return FALSE;
+
+	return TRUE;
+
+}
+
+icaltimezone *
+e_cal_backend_cache_get_default_timezone (ECalBackendCache *cache)
+{
+	icaltimezone *zone;
+	const char *comp_str;
+	ECalBackendCachePrivate *priv;
+
+	g_return_val_if_fail (E_IS_CAL_BACKEND_CACHE (cache), NULL);
+
+	priv = cache->priv;
+
+	/*  look for the timezone in the cache */
+	comp_str = e_file_cache_get_object (E_FILE_CACHE (cache), "default_zone");
+	if (comp_str) {
+		icalcomponent *icalcomp;
+
+		icalcomp = icalparser_parse_string (comp_str);
+		if (icalcomp) {
+			zone = icaltimezone_new ();
+			if (icaltimezone_set_component (zone, icalcomp) == 1) {
+				return zone;
+			} else {
+				icalcomponent_free (icalcomp);
+				icaltimezone_free (zone, 1);
+			}
+		}
+	}
+
+	return NULL;
+}
+
+gboolean
+e_cal_backend_cache_remove_timezone (ECalBackendCache *cache, const char *tzid)
+{
+	gpointer orig_key, orig_value;
+	ECalBackendCachePrivate *priv;
+
+	g_return_val_if_fail (E_IS_CAL_BACKEND_CACHE (cache), FALSE);
+	g_return_val_if_fail (tzid != NULL, FALSE);
+
+	priv = cache->priv;
+
+	if (g_hash_table_lookup_extended (priv->timezones, tzid, &orig_key, &orig_value)) {
+		g_hash_table_remove (priv->timezones, tzid);
+		g_free (orig_key);
+		icaltimezone_free (orig_value, 1);
+	}
+
+	return e_file_cache_remove_object (E_FILE_CACHE (cache), tzid);
+}
+
+GSList *
+e_cal_backend_cache_get_keys (ECalBackendCache *cache)
+{
+        /* return null if cache is not a valid Backend Cache.  */
+	g_return_val_if_fail (E_IS_CAL_BACKEND_CACHE (cache), NULL);
+        return e_file_cache_get_keys (E_FILE_CACHE (cache));
+}
+
+void
+e_cal_backend_cache_set_marker (ECalBackendCache *cache)
+{
+	g_return_if_fail (E_IS_CAL_BACKEND_CACHE (cache));
+	e_file_cache_add_object (E_FILE_CACHE (cache), "populated", "TRUE");
+}
+
+const char *
+e_cal_backend_cache_get_marker (ECalBackendCache *cache)
+{
+	g_return_val_if_fail (E_IS_CAL_BACKEND_CACHE (cache), NULL);
+	if (e_file_cache_get_object (E_FILE_CACHE (cache), "populated"))
+		return "";
+	return NULL;
+}
